@@ -1,15 +1,15 @@
 """nbconvert version info."""
+
 import re
-from typing import List
 
 # Version string must appear intact for versioning
-__version__ = "7.8.0"
+__version__ = "7.17.0"
 
 # Build up version_info tuple for backwards compatibility
 pattern = r"(?P<major>\d+).(?P<minor>\d+).(?P<patch>\d+)(?P<rest>.*)"
 match = re.match(pattern, __version__)
-assert match is not None  # noqa
-parts: List[object] = [int(match[part]) for part in ["major", "minor", "patch"]]
+assert match is not None
+parts: list[object] = [int(match[part]) for part in ["major", "minor", "patch"]]
 if match["rest"]:
     parts.append(match["rest"])
 version_info = tuple(parts)
