@@ -1,13 +1,11 @@
 # Class base Preprocessors
-# Backwards compatability for imported name
+# Backwards compatibility for imported name
 from nbclient.exceptions import CellExecutionError
 
 from .base import Preprocessor
 from .clearmetadata import ClearMetadataPreprocessor
 from .clearoutput import ClearOutputPreprocessor
-
-# decorated function Preprocessors
-from .coalescestreams import coalesce_streams
+from .coalescestreams import CoalesceStreamsPreprocessor
 from .convertfigures import ConvertFiguresPreprocessor
 from .csshtmlheader import CSSHTMLHeaderPreprocessor
 from .execute import ExecutePreprocessor
@@ -18,3 +16,21 @@ from .latex import LatexPreprocessor
 from .regexremove import RegexRemovePreprocessor
 from .svg2pdf import SVG2PDFPreprocessor
 from .tagremove import TagRemovePreprocessor
+
+__all__ = [
+    "CSSHTMLHeaderPreprocessor",
+    "CellExecutionError",
+    "ClearMetadataPreprocessor",
+    "ClearOutputPreprocessor",
+    "CoalesceStreamsPreprocessor",
+    "ConvertFiguresPreprocessor",
+    "ExecutePreprocessor",
+    "ExtractAttachmentsPreprocessor",
+    "ExtractOutputPreprocessor",
+    "HighlightMagicsPreprocessor",
+    "LatexPreprocessor",
+    "Preprocessor",
+    "RegexRemovePreprocessor",
+    "SVG2PDFPreprocessor",
+    "TagRemovePreprocessor",
+]
